@@ -30,21 +30,23 @@ from ducktape.tests.result import TestResult, IGNORE, PASS, FAIL
 from ducktape.utils.local_filesystem_utils import mkdir_p
 
 
-def run_client(server_hostname, server_port, test_id, test_index, logger_name, log_dir, debug):
-    client = RunnerClient(server_hostname, server_port, test_id, test_index, logger_name, log_dir, debug)
+def run_client(*args, **kwargs):
+    client = RunnerClient(*args, **kwargs)
     client.run()
 
 
 class RunnerClient(object):
     """Run a single test"""
 
-    def __init__(self, server_hostname, server_port, test_id, test_index, logger_name, log_dir, debug):
+    def __init__(self, server_hostname, server_port, test_id,
+                 test_index, logger_name, log_dir, debug, fail_bad_cluster_utilization):
         signal.signal(signal.SIGTERM, self._sigterm_handler)  # register a SIGTERM handler
 
         self.serde = SerDe()
         self.logger = test_logger(logger_name, log_dir, debug)
         self.runner_port = server_port
 
+        self.fail_bad_cluster_utilization = fail_bad_cluster_utilization
         self.test_id = test_id
         self.test_index = test_index
         self.id = "test-runner-%d-%d" % (os.getpid(), id(self))
@@ -98,9 +100,6 @@ class RunnerClient(object):
             self.send(self.message.finished(result=result))
             return
 
-        # Results from this test, as well as logs will be dumped here
-        mkdir_p(TestContext.results_dir(self.test_context, self.test_index))
-
         start_time = -1
         stop_time = -1
         test_status = PASS
@@ -108,6 +107,8 @@ class RunnerClient(object):
         data = None
 
         try:
+            # Results from this test, as well as logs will be dumped here
+            mkdir_p(TestContext.results_dir(self.test_context, self.test_index))
             # Instantiate test
             self.test = self.test_context.cls(self.test_context)
 
@@ -137,11 +138,11 @@ class RunnerClient(object):
             self.log(logging.INFO, "PASS")
 
         except BaseException as e:
-            err_trace = str(e) + "\n" + traceback.format_exc(limit=16)
-            self.log(logging.INFO, "FAIL: " + err_trace)
-
+            # mark the test as failed before doing anything else
             test_status = FAIL
+            err_trace = self._exc_msg(e)
             summary += err_trace
+            self.log(logging.INFO, "FAIL: " + err_trace)
 
         finally:
             self.teardown_test(teardown_services=not self.session_context.no_teardown, test_status=test_status)
@@ -153,6 +154,8 @@ class RunnerClient(object):
                 if service_errors:
                     summary += "\n\n" + service_errors
 
+            test_status, summary = self._check_cluster_utilization(test_status, summary)
+
             result = TestResult(
                 self.test_context,
                 self.test_index,
@@ -167,15 +170,35 @@ class RunnerClient(object):
             self.log(logging.INFO, "Data: %s" % str(result.data))
 
             result.report()
+            # Tell the server we are finished
+            self._do_safely(lambda: self.send(self.message.finished(result=result)),
+                            "Problem sending FINISHED message for " + str(self.test_metadata) + ":\n")
+            # Release test_context resources only after creating the result and finishing logging activity
+            # The Sender object uses the same logger, so we postpone closing until after the finished message is sent
+            self.test_context.close()
+            self.test_context = None
+            self.test = None
 
-        # Tell the server we are finished
-        self._do_safely(lambda: self.send(self.message.finished(result=result)), "Problem sending FINISHED message:")
+    def _check_cluster_utilization(self, result, summary):
+        """Checks if the number of nodes used by a test is less than the number of
+        nodes requested by the test. If this is the case and we wish to fail
+        on bad cluster utilization, the result value is failed. Will also print
+        a warning if the test passes and the node utilization doesn't match.
+        """
+        max_used = self.cluster.max_used()
+        total = len(self.cluster.all())
+        if max_used < total:
+            message = "Test requested %d nodes, used only %d" % (total, max_used)
+            if self.fail_bad_cluster_utilization:
+                # only check node utilization on test pass
+                if result == PASS:
+                    self.log(logging.INFO, "FAIL: " + message)
 
-        # Release test_context resources only after creating the result and finishing logging activity
-        # The Sender object uses the same logger, so we postpone closing until after the finished message is sent
-        self.test_context.close()
-        self.test_context = None
-        self.test = None
+                result = FAIL
+                summary += message
+            else:
+                self.log(logging.WARN, message)
+        return result, summary
 
     def setup_test(self):
         """start services etc"""
@@ -191,11 +214,14 @@ class RunnerClient(object):
         self.log(logging.INFO, "Running...")
         return self.test_context.function(self.test)
 
+    def _exc_msg(self, e):
+        return repr(e) + "\n" + traceback.format_exc(limit=16)
+
     def _do_safely(self, action, err_msg):
         try:
             action()
         except BaseException as e:
-            self.log(logging.WARN, err_msg + " " + str(e) + "\n" + traceback.format_exc(limit=16))
+            self.log(logging.WARN, err_msg + " " + self._exc_msg(e))
 
     def teardown_test(self, teardown_services=True, test_status=None):
         """teardown method which stops services, gathers log data, removes persistent state, and releases cluster nodes.
